//
//  IDmeCredential.h
//  ID.me Verify
//
//  Created by Arthur Sabintsev on 7/8/13.
//  Copyright (c) 2013 ID.me, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

/// Plain text string for the @c IDmeVerifyCredentialTypeUnknown type in the @c IDmeVerifyCredentialType typedef
extern NSString *const kIDmeVerifyCredentialTypeNameUnknown;

/// Plain text string for the @c IDmeVerifyCredentialTypeStateLicense type in the @c IDmeVerifyCredentialType typedef
extern NSString *const kIDmeVerifyCredentialTypeNameStateLicense;

/// Plain text string for the @c IDmeVerifyCredentialTypeCommonAccessCard type in the @c IDmeVerifyCredentialType typedef
extern NSString *const kIDmeVerifyCredentialTypeNameCommonAccessCard;

/// Plain text string for the @c IDmeVerifyCredentialTypeUniformedServicesCard type in the @c IDmeVerifyCredentialType typedef
extern NSString *const kIDmeVerifyCredentialTypeNameUniformedServicesCard;

/// Plain text string for the @c IDmeVerifyCredentialTypeUniformedServicesDependentCard type in the @c IDmeVerifyCredentialType typedef
extern NSString *const kIDmeVerifyCredentialTypeNameUniformedServicesDependentCard;

typedef NS_ENUM(NSUInteger, IDmeVerifyCredentialType)
{
    /// Scanned Credential is an unknown credential, and therefore, can not be parsed.
    IDmeVerifyCredentialTypeUnknown = 0,
    
    /// Scanned Credential is a U.S. State License (including District of Columbia).
    IDmeVerifyCredentialTypeStateLicense,
    
    /// Scanned Credential is a Common Access Card (aka CAC).
    IDmeVerifyCredentialTypeCommonAccessCard,
    
    /// Scanned Credential is a Uniformed Services Card owned by a Sponsor
    IDmeVerifyCredentialTypeUniformedServicesCard,
    
    /// Scanned Credential is a Uniformed Services Card owned by a Dependent
    IDmeVerifyCredentialTypeUniformedServicesDependentCard
};

@interface IDmeCredential : NSObject

@property (nonatomic, assign, readonly) IDmeVerifyCredentialType credentialType;
@property (nonatomic, copy, readonly) NSString *credentialTypeName;
@property (nonatomic, copy) NSString *barcode;
@property (nonatomic, copy) NSString *firstName;
@property (nonatomic, copy) NSString *lastName;
@property (nonatomic, copy) NSString *address;
@property (nonatomic, copy) NSString *birthDay;
@property (nonatomic, copy) NSString *birthMonth;
@property (nonatomic, copy) NSString *birthYear;
@property (nonatomic, copy) NSString *birthdate;
@property (nonatomic, copy) NSString *age;

- (instancetype)initWithBarcode:(NSString *)barcode andCredentialType:(IDmeVerifyCredentialType)credentialType;

@end
