//
//  ViewController.m
//  ID.me Sample Project
//
//  Created by Arthur Sabintsev on 10/7/13.
//  Copyright (c) 2013 ID.me. All rights reserved.
//

#import "ViewController.h"
#import "IDmeVerify.h"

#define IDME_ALERTVIEW_TAG  1000

@interface ViewController () <UIAlertViewDelegate>

- (IBAction)scanAction:(id)sender;
- (IBAction)verifyAction:(id)sender;

@end

@implementation ViewController

#pragma mark - Button Actions
- (void)scanAction:(id)sender
{
    [[IDmeVerify sharedInstance] scanCredentialInViewController:self
                                             forCredentialTypes:IDmeVerifyDetectCredentialTypeAll
                                                    withResults:^(IDmeCredential *credential, NSError *error) {
                                                        
                                                        if (error) { // Error
                                                            
                                                            NSLog(@"Scan Error %d: %@", error.code, error.localizedDescription);
                                                            
                                                        } else { // Credential was scanned and parsed successfully
                                                            
                                                            switch ([credential credentialType]) {
                                                                    
                                                                case IDmeVerifyCredentialTypeStateLicense:{
                                                                    IDmeStateLicense *stateLicense = (IDmeStateLicense *)credential;
                                                                    NSLog(@"%@", [stateLicense description]);
                                                                    // Extract data from stateLicense
                                                                } break;
                                                                    
                                                                case IDmeVerifyCredentialTypeCommonAccessCard:{
                                                                    IDmeCommonAccessCard *commonAccessCard = (IDmeCommonAccessCard *)credential;
                                                                    NSLog(@"%@", [commonAccessCard description]);
                                                                    // Extract data from commonAccessCard
                                                                } break;
                                                                    
                                                                case IDmeVerifyCredentialTypeUniformedServicesCard:{
                                                                    IDmeUniformedServicesCard *uniformedServicesCard = (IDmeUniformedServicesCard *)credential;
                                                                    NSLog(@"%@", [uniformedServicesCard description]);
                                                                    // Extract data from uniformedServicesCard
                                                                } break;
                                                                    
                                                                case IDmeVerifyCredentialTypeUniformedServicesDependentCard:{
                                                                    IDmeUniformedServicesDependentCard *uniformedServicesDependentCard = (IDmeUniformedServicesDependentCard *)credential;
                                                                    NSLog(@"%@", [uniformedServicesDependentCard description]);
                                                                    // Extract data from uniformedServicesDependentCard
                                                                } break;
                                                                    
                                                                case IDmeVerifyCredentialTypeUnknown:{
                                                                    // Occurs when scanned credential is unknown
                                                                    // An error is also thrown, so this conditional should never be entered if you use the if-else block in the aformenetioned code snippet.
                                                                } break;
                                                            }
                                                            
                                                        }
                                                    }]; 
}

- (void)verifyAction:(id)sender
{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Verify Affiliation"
                                                        message:@"Which affiliation would you like to verify?"
                                                       delegate:self cancelButtonTitle:@"Cancel"
                                              otherButtonTitles:@"Military", @"Student", @"First Responder", nil];
    [alertView setTag:IDME_ALERTVIEW_TAG];
    [alertView show];
}

#pragma mark - UIAlertViewDelegate Methods
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == IDME_ALERTVIEW_TAG) {
        
        IDmeVerifyAffiliationType affiliationType = IDmeVerifyAffiliationTypeMilitary;
        switch (buttonIndex) {
            case 1: // Military
                affiliationType = IDmeVerifyAffiliationTypeMilitary;
                break;
                
            case 2: // Student
                affiliationType = IDmeVerifyAffiliationTypeStudent;
                break;
                
            case 3: // First Responder
                affiliationType = IDmeVerifyAffiliationTypeResponder;
                break;
        }
        
        if (buttonIndex > 0) {
#warning The clientID and redirectURI should only be used in this sample project. Please obtain your own clientID and redirectURI from http://developer.sandbox.id.me
            [[IDmeVerify sharedInstance] verifyUserInViewController:self
                                                       withClientID:@"be4da8f971329f362"
                                                        redirectURI:@"https://developer.sandbox.id.me"
                                                    affiliationType:affiliationType
                                                      inSandboxMode:YES
                                                        withResults:^(NSDictionary *userProfile, NSError *error) {
                                                            
                                                            if (error) { // Error
                                                                
                                                                NSLog(@"Verification Error %d: %@", error.code, error.localizedDescription);
                                                                
                                                            } else { // Verification was successful
                                                                
                                                                NSLog(@"\nVerification Results:\n %@", userProfile);
                                                                
                                                            }
                                                            
                                                        }];
        }
    }
}


#pragma mark - Orientation Methods
- (NSUInteger)supportedInterfaceOrientations
{
    return UIInterfaceOrientationMaskPortrait;
}

- (BOOL)shouldAutorotate
{
    return YES;
}

@end
